@@ -11,9 +11,11 @@
 import os.path as osp
 from unittest.mock import MagicMock, Mock
 
+from spyder.api.plugins import Plugins
+from spyder.utils.qthelpers import qapplication
+
 # This is needed to avoid an error because QtAwesome
 # needs a QApplication to work correctly.
-from spyder.utils.qthelpers import qapplication
 app = qapplication()
 
 from qtpy.QtWidgets import QMainWindow
@@ -41,20 +43,18 @@ def editor_plugin(qtbot, monkeypatch):
         def __getattr__(self, attr):
             if attr.endswith('actions'):
                 return []
-            elif attr == 'projects':
-                projects = Mock()
-                projects.get_active_project.return_value = None
-                return projects
-            elif attr == 'ipyconsole':
-                ipyconsole = Mock()
-                ipyconsole.get_pdb_state.return_value = False
-                return ipyconsole
             else:
                 return Mock()
 
         def get_spyder_pythonpath(*args):
             return []
 
+        def get_plugin(self, plugin_name, error=True):
+            if plugin_name in [Plugins.IPythonConsole, Plugins.Projects]:
+                return None
+            else:
+                return Mock()
+
     window = MainMock()
     editor = Editor(window)
     window.setCentralWidget(editor)
