@@ -18,10 +18,13 @@ from qtpy.QtWidgets import QApplication, QDesktopWidget, QDockWidget
 # Local imports
 from spyder.api.exceptions import SpyderAPIError
 from spyder.api.plugins import Plugins, SpyderPluginV2
+from spyder.api.plugin_registration.decorators import (
+    on_plugin_available, on_plugin_teardown)
 from spyder.api.translations import get_translation
 from spyder.api.utils import get_class_values
 from spyder.plugins.mainmenu.api import ApplicationMenus, ViewMenuSections
-from spyder.plugins.layout.container import LayoutContainer
+from spyder.plugins.layout.container import (
+    LayoutContainer, LayoutContainerActions, LayoutPluginMenus)
 from spyder.plugins.layout.layouts import (DefaultLayouts,
                                            HorizontalSplitLayout,
                                            MatlabLayout, RLayout,
@@ -31,14 +34,32 @@ from spyder.plugins.toolbar.api import (
     ApplicationToolbars, MainToolbarSections)
 from spyder.py3compat import qbytearray_to_str  # FIXME:
 
+
 # Localization
 _ = get_translation("spyder")
 
 # Constants
+
 # Number of default layouts available
 DEFAULT_LAYOUTS = get_class_values(DefaultLayouts)
-# Version passed to saveState/restoreState
-WINDOW_STATE_VERSION = 1
+
+# ----------------------------------------------------------------------------
+# ---- Window state version passed to saveState/restoreState.
+# ----------------------------------------------------------------------------
+# This defines the layout version used by different Spyder releases. In case
+# there's a need to reset the layout when moving from one release to another,
+# please increase the number below in integer steps, e.g. from 1 to 2, and
+# leave a mention below explaining what prompted the change.
+#
+# The current versions are:
+#
+# * Spyder 4: Version 0 (it was the default).
+# * Spyder 5.0.0 to 5.0.5: Version 1 (a bump was required due to the new API).
+# * Spyder 5.1.0: Version 2 (a bump was required due to the migration of
+#                            Projects to the new API).
+# * Spyder 5.2.0: Version 3 (a bump was required due to the migration of
+#                            IPython Console to the new API)
+WINDOW_STATE_VERSION = 3
 
 
 class Layout(SpyderPluginV2):
@@ -50,10 +71,12 @@ class Layout(SpyderPluginV2):
     REQUIRES = [Plugins.All]  # Uses wildcard to require all the plugins
     CONF_FILE = False
     CONTAINER_CLASS = LayoutContainer
+    CAN_BE_DISABLED = False
 
     # --- SpyderDockablePlugin API
     # ------------------------------------------------------------------------
-    def get_name(self):
+    @staticmethod
+    def get_name():
         return _("Layout")
 
     def get_description(self):
@@ -62,8 +85,7 @@ class Layout(SpyderPluginV2):
     def get_icon(self):
         return self.create_icon("history")  # FIXME:
 
-    def register(self):
-        container = self.get_container()
+    def on_initialize(self):
         self._last_plugin = None
         self._first_spyder_run = False
         self._fullscreen_flag = None
@@ -83,54 +105,90 @@ class Layout(SpyderPluginV2):
         self.register_layout(self, HorizontalSplitLayout)
         self.register_layout(self, VerticalSplitLayout)
 
-        mainmenu = self.get_plugin(Plugins.MainMenu)
-        if mainmenu:
-            # Add Panes related actions to View application menu
-            panes_items = [
-                container._plugins_menu,
-                container._lock_interface_action,
-                container._close_dockwidget_action,
-                container._maximize_dockwidget_action]
-            for panes_item in panes_items:
-                mainmenu.add_item_to_application_menu(
-                    panes_item,
-                    menu_id=ApplicationMenus.View,
-                    section=ViewMenuSections.Pane,
-                    before_section=ViewMenuSections.Toolbar)
-            # Add layouts menu to View application menu
-            layout_items = [
-                container._layouts_menu,
-                container._toggle_next_layout_action,
-                container._toggle_previous_layout_action]
-            for layout_item in layout_items:
-                mainmenu.add_item_to_application_menu(
-                    layout_item,
-                    menu_id=ApplicationMenus.View,
-                    section=ViewMenuSections.Layout)
-            # Add fullscreen action to View application menu
-            mainmenu.add_item_to_application_menu(
-                container._fullscreen_action,
-                menu_id=ApplicationMenus.View,
-                section=ViewMenuSections.Bottom)
-
-        toolbars = self.get_plugin(Plugins.Toolbar)
-        if toolbars:
-            # Add actions to Main application toolbar
-            before_action = self.get_action(
-                PreferencesActions.Show,
-                plugin=Plugins.Preferences
-            )
-
-            toolbars.add_item_to_application_toolbar(
-                container._maximize_dockwidget_action,
-                toolbar_id=ApplicationToolbars.Main,
-                section=MainToolbarSections.ApplicationSection,
-                before=before_action
-            )
-
-        # Update actions icons and text
         self._update_fullscreen_action()
 
+    @on_plugin_available(plugin=Plugins.MainMenu)
+    def on_main_menu_available(self):
+        mainmenu = self.get_plugin(Plugins.MainMenu)
+        container = self.get_container()
+        # Add Panes related actions to View application menu
+        panes_items = [
+            container._plugins_menu,
+            container._lock_interface_action,
+            container._close_dockwidget_action,
+            container._maximize_dockwidget_action]
+        for panes_item in panes_items:
+            mainmenu.add_item_to_application_menu(
+                panes_item,
+                menu_id=ApplicationMenus.View,
+                section=ViewMenuSections.Pane,
+                before_section=ViewMenuSections.Toolbar)
+        # Add layouts menu to View application menu
+        layout_items = [
+            container._layouts_menu,
+            container._toggle_next_layout_action,
+            container._toggle_previous_layout_action]
+        for layout_item in layout_items:
+            mainmenu.add_item_to_application_menu(
+                layout_item,
+                menu_id=ApplicationMenus.View,
+                section=ViewMenuSections.Layout,
+                before_section=ViewMenuSections.Bottom)
+        # Add fullscreen action to View application menu
+        mainmenu.add_item_to_application_menu(
+            container._fullscreen_action,
+            menu_id=ApplicationMenus.View,
+            section=ViewMenuSections.Bottom)
+
+    @on_plugin_available(plugin=Plugins.Toolbar)
+    def on_toolbar_available(self):
+        container = self.get_container()
+        toolbars = self.get_plugin(Plugins.Toolbar)
+        # Add actions to Main application toolbar
+        toolbars.add_item_to_application_toolbar(
+            container._maximize_dockwidget_action,
+            toolbar_id=ApplicationToolbars.Main,
+            section=MainToolbarSections.ApplicationSection,
+            before=PreferencesActions.Show
+        )
+
+    @on_plugin_teardown(plugin=Plugins.MainMenu)
+    def on_main_menu_teardown(self):
+        mainmenu = self.get_plugin(Plugins.MainMenu)
+        # Remove Panes related actions from the View application menu
+        panes_items = [
+            LayoutPluginMenus.PluginsMenu,
+            LayoutContainerActions.LockDockwidgetsAndToolbars,
+            LayoutContainerActions.CloseCurrentDockwidget,
+            LayoutContainerActions.MaximizeCurrentDockwidget]
+        for panes_item in panes_items:
+            mainmenu.remove_item_from_application_menu(
+                panes_item,
+                menu_id=ApplicationMenus.View)
+        # Remove layouts menu from the View application menu
+        layout_items = [
+            LayoutPluginMenus.LayoutsMenu,
+            LayoutContainerActions.NextLayout,
+            LayoutContainerActions.PreviousLayout]
+        for layout_item in layout_items:
+            mainmenu.remove_item_from_application_menu(
+                layout_item,
+                menu_id=ApplicationMenus.View)
+        # Remove fullscreen action from the View application menu
+        mainmenu.remove_item_from_application_menu(
+            LayoutContainerActions.Fullscreen,
+            menu_id=ApplicationMenus.View)
+
+    @on_plugin_teardown(plugin=Plugins.Toolbar)
+    def on_toolbar_teardown(self):
+        toolbars = self.get_plugin(Plugins.Toolbar)
+
+        # Remove actions from the Main application toolbar
+        toolbars.remove_item_from_application_toolbar(
+            LayoutContainerActions.MaximizeCurrentDockwidget,
+            toolbar_id=ApplicationToolbars.Main
+        )
+
     def before_mainwindow_visible(self):
         # Update layout menu
         self.update_layout_menu_actions()
@@ -715,6 +773,7 @@ class Layout(SpyderPluginV2):
             except AttributeError:
                 # Old API
                 action = plugin._toggle_view_action
+                action.action_id = f'switch to {plugin.CONF_SECTION}'
 
             if action:
                 action.setChecked(plugin.dockwidget.isVisible())
