@@ -12,8 +12,6 @@ import time
 # Local imports
 from spyder.api.plugins import Plugins
 from spyder.api.plugin_registration.registry import PLUGIN_REGISTRY
-from spyder.config.manager import CONF
-from spyder.plugins.preferences.api import PreferencePages
 from spyder.plugins.maininterpreter.plugin import MainInterpreter
 from spyder.plugins.preferences.tests.conftest import MainWindowMock
 from spyder.utils.conda import get_list_conda_envs
@@ -22,7 +20,7 @@ from spyder.utils.pyenv import get_list_pyenv_envs
 
 # Get envs to show them in the Main interpreter page. This is actually
 # done in a thread in the InterpreterStatus widget.
-# We also recording the time needed to get them to compare it with the
+# We're also recording the time needed to get them to compare it with the
 # loading time of that config page.
 t0 = time.time()
 get_list_conda_envs()
@@ -31,14 +29,11 @@ GET_ENVS_TIME = time.time() - t0
 
 
 def test_load_time(qtbot):
-    from spyder.plugins.maininterpreter.confpage import (
-        MainInterpreterConfigPage)
-
     # Create Preferences dialog
     main = MainWindowMock()
     preferences = main.get_plugin(Plugins.Preferences)
 
-    main_interpreter = PLUGIN_REGISTRY.register_plugin(main, MainInterpreter)
+    PLUGIN_REGISTRY.register_plugin(main, MainInterpreter)
 
     # Create page and measure time to do it
     t0 = time.time()
