@@ -14,7 +14,6 @@ import yaml
 
 # Local imports
 from spyder.dependencies import DESCRIPTIONS, OPTIONAL
-from spyder.py3compat import PY2
 
 # Constants
 HERE = os.path.abspath(os.path.dirname(__file__))
@@ -205,11 +204,6 @@ def test_dependencies_for_binder_in_sync():
     if 'pytest-xvfb' in spyder_env:
         spyder_env.pop('pytest-xvfb')
 
-    # There's no need to test for this because we install it
-    # from master in some cases.
-    for req in [spyder_env, spyder_reqs]:
-        req.pop('python-lsp-server')
-
     # Check that the requirement files match the environment yaml file
     full_reqs = {}
     full_reqs.update(test_reqs)
@@ -225,22 +219,9 @@ def test_dependencies_for_spyder_dialog_in_sync():
     spyder_deps = parse_spyder_dependencies()
     spyder_reqs = parse_requirements(REQ_FPATH)
 
-    # No need to check for these deps because either we're using
-    # a subrepo for them or we're installing them from master.
-    for req in [spyder_deps, spyder_reqs]:
-        req.pop('spyder-kernels')
-        req.pop('python-lsp-server')
-
     if 'pyqt' in spyder_reqs:
         spyder_reqs.pop('pyqt')
 
-    if PY2:
-        if 'ipython' in spyder_reqs:
-            spyder_reqs.pop('ipython')
-
-        if 'ipython' in spyder_deps:
-            spyder_deps.pop('ipython')
-
     assert spyder_deps == spyder_reqs
 
 
@@ -251,12 +232,6 @@ def test_dependencies_for_spyder_setup_install_requires_in_sync():
     spyder_setup = parse_setup_install_requires(SETUP_FPATH)
     spyder_reqs = parse_requirements(REQ_FPATH)
 
-    # No need to check for these deps because either we're using
-    # a subrepo for them or we're installing them from master.
-    for req in [spyder_reqs, spyder_setup]:
-        req.pop('spyder-kernels')
-        req.pop('python-lsp-server')
-
     if 'pyqtwebengine' in spyder_setup:
         spyder_setup.pop('pyqtwebengine')
 
