@@ -6,18 +6,18 @@
 
 """Tests for pyenv.py"""
 
-import os
+import sys
 import time
 
 import pytest
 
 from spyder.config.base import running_in_ci
-from spyder.utils.pyenv import (
-    get_list_pyenv_envs, get_list_pyenv_envs_cache)
+from spyder.utils.pyenv import get_list_pyenv_envs, get_list_pyenv_envs_cache
 
 
 @pytest.mark.skipif(not running_in_ci(), reason="Only meant for CIs")
-@pytest.mark.skipif(os.name == 'nt', reason="Doesn't work on Windows")
+@pytest.mark.skipif(not sys.platform.startswith('linux'),
+                    reason="Only runs on Linux")
 def test_get_list_pyenv_envs():
     output = get_list_pyenv_envs()
     expected_envs = ['pyenv: 3.8.1']
@@ -25,7 +25,8 @@ def test_get_list_pyenv_envs():
 
 
 @pytest.mark.skipif(not running_in_ci(), reason="Only meant for CIs")
-@pytest.mark.skipif(os.name == 'nt', reason="Doesn't work on Windows")
+@pytest.mark.skipif(not sys.platform.startswith('linux'),
+                    reason="Only runs on Linux")
 def test_get_list_pyenv_envs_cache():
     time0 = time.time()
     output = get_list_pyenv_envs_cache()
