@@ -21,37 +21,47 @@ import sys
 import textwrap
 
 # Third party imports
-import qdarkstyle
-from qtpy.QtCore import QPoint, Qt
+from qtpy.QtCore import QPoint, QRegularExpression, Qt
 from qtpy.QtGui import QCursor, QTextCursor, QTextDocument
 from qtpy.QtWidgets import QApplication
-from qtpy import QT_VERSION
 from spyder_kernels.utils.dochelpers import (getargspecfromtext, getobj,
                                              getsignaturefromtext)
 
 # Local imports
-from spyder.config.gui import is_dark_interface
 from spyder.config.manager import CONF
 from spyder.py3compat import is_text_string, to_text_string
-from spyder.utils import encoding, sourcecode, programs
+from spyder.utils import encoding, sourcecode
 from spyder.utils import syntaxhighlighters as sh
 from spyder.utils.misc import get_error_match
+from spyder.utils.palette import QStylePalette
 from spyder.widgets.arraybuilder import ArrayBuilderDialog
 
-QT55_VERSION = programs.check_version(QT_VERSION, "5.5", ">=")
 
-if QT55_VERSION:
-    from qtpy.QtCore import QRegularExpression
-else:
-    from qtpy.QtCore import QRegExp
+# List of possible EOL symbols
+EOL_SYMBOLS = [
+    # Put first as it correspond to a single line return
+    "\r\n",  # Carriage Return + Line Feed
+    "\r",  # Carriage Return
+    "\n",  # Line Feed
+    "\v",  # Line Tabulation
+    "\x0b",  # Line Tabulation
+    "\f",  # Form Feed
+    "\x0c",   # Form Feed
+    "\x1c",   # File Separator
+    "\x1d",   # Group Separator
+    "\x1e",   # Record Separator
+    "\x85",   # Next Line (C1 Control Code)
+    "\u2028",   # Line Separator
+    "\u2029",   # Paragraph Separator
+]
 
 
 class BaseEditMixin(object):
 
-    _PARAMETER_HIGHLIGHT_COLOR = '#DAA520'
-    _DEFAULT_TITLE_COLOR = '#2D62FF'
-    _CHAR_HIGHLIGHT_COLOR = 'red'
-    _DEFAULT_TEXT_COLOR = '#999999'
+    _PARAMETER_HIGHLIGHT_COLOR = QStylePalette.COLOR_ACCENT_4
+    _DEFAULT_TITLE_COLOR = QStylePalette.COLOR_ACCENT_4
+    _CHAR_HIGHLIGHT_COLOR = QStylePalette.COLOR_ACCENT_4
+    _DEFAULT_TEXT_COLOR = QStylePalette.COLOR_TEXT_2
     _DEFAULT_LANGUAGE = 'python'
     _DEFAULT_MAX_LINES = 10
     _DEFAULT_MAX_WIDTH = 60
@@ -96,16 +106,18 @@ class BaseEditMixin(object):
         if at_point is not None:
             # Showing tooltip at point position
             margin = (self.document().documentMargin() / 2) + 1
-            cx, cy = at_point.x() - margin, at_point.y() - margin
+            cx = int(at_point.x() - margin)
+            cy = int(at_point.y() - margin)
         elif at_line is not None:
             # Showing tooltip at line
             cx = 5
             line = at_line - 1
             cursor = QTextCursor(self.document().findBlockByNumber(line))
-            cy = self.cursorRect(cursor).top()
+            cy = int(self.cursorRect(cursor).top())
         else:
             # Showing tooltip at cursor position
             cx, cy = self.get_coordinates('cursor')
+            cx = int(cx)
             cy = int(cy - font.pointSize() / 2)
 
         # Calculate vertical delta
@@ -132,21 +144,16 @@ class BaseEditMixin(object):
         if id(widget) in self._styled_widgets:
             return
         self._styled_widgets.add(id(widget))
-
-        if is_dark_interface():
-            css = qdarkstyle.load_stylesheet_from_environment()
-            widget.setStyleSheet(css)
-            palette = widget.palette()
-            background = palette.color(palette.Window).lighter(150).name()
-            border = palette.color(palette.Window).lighter(200).name()
-            name = widget.__class__.__name__
-            widget.setObjectName(name)
-            extra_css = '''
-                {0}#{0} {{
-                    background-color:{1};
-                    border: 1px solid {2};
-                }}'''.format(name, background, border)
-            widget.setStyleSheet(css + extra_css)
+        background = QStylePalette.COLOR_BACKGROUND_4
+        border = QStylePalette.COLOR_TEXT_4
+        name = widget.__class__.__name__
+        widget.setObjectName(name)
+        css = '''
+            {0}#{0} {{
+                background-color:{1};
+                border: 1px solid {2};
+            }}'''.format(name, background, border)
+        widget.setStyleSheet(css)
 
     def _get_inspect_shortcut(self):
         """
@@ -297,7 +304,8 @@ class BaseEditMixin(object):
                 shortcut = self._get_inspect_shortcut()
                 if shortcut:
                     base_style = (
-                        'background-color:#fafbfc;color:#444d56;'
+                        f'background-color:{QStylePalette.COLOR_BACKGROUND_4};'
+                        f'color:{QStylePalette.COLOR_TEXT_1};'
                         'font-size:11px;'
                     )
                     help_text = ''
@@ -314,7 +322,8 @@ class BaseEditMixin(object):
                 template += (
                     '<hr>'
                     '<div align="left">'
-                    '<span style="color:#148CD2;text-decoration:none;'
+                    f'<span style="color: {QStylePalette.COLOR_ACCENT_4};'
+                    'text-decoration:none;'
                     'font-family:"{font_family}";font-size:{size}pt;><i>'
                     ''.format(font_family=font_family,
                               size=text_size)
@@ -458,7 +467,11 @@ class BaseEditMixin(object):
         signature_or_text = signature_or_text.replace('{', '&#123;')
         signature_or_text = signature_or_text.replace('}', '&#125;')
 
-        lines = signature_or_text.split('\n')
+        # Remove 'ufunc' signature if needed. See spyder-ide/spyder#11821
+        lines = [line for line in signature_or_text.split('\n')
+                 if 'ufunc' not in line]
+        signature_or_text = '\n'.join(lines)
+
         if language == 'python':
             open_func_char = '('
             has_multisignature = False
@@ -664,6 +677,10 @@ class BaseEditMixin(object):
         self._last_point = None
         self.tooltip_widget.hide()
 
+    # ----- Required methods for the LSP
+    def document_did_change(self, text=None):
+        pass
+
     #------EOL characters
     def set_eol_chars(self, text):
         """Set widget end-of-line (EOL) characters from text (analyzes text)"""
@@ -676,6 +693,7 @@ class BaseEditMixin(object):
             self.document().setModified(True)
             if self.sig_eol_chars_changed is not None:
                 self.sig_eol_chars_changed.emit(eol_chars)
+            self.document_did_change(text)
 
     def get_line_separator(self):
         """Return line separator based on current EOL mode"""
@@ -685,15 +703,15 @@ class BaseEditMixin(object):
             return os.linesep
 
     def get_text_with_eol(self):
-        """Same as 'toPlainText', replace '\n'
-        by correct end-of-line characters"""
-        utext = to_text_string(self.toPlainText())
-        lines = utext.splitlines()
+        """
+        Same as 'toPlainText', replacing '\n' by correct end-of-line
+        characters.
+        """
+        text = self.toPlainText()
         linesep = self.get_line_separator()
-        txt = linesep.join(lines)
-        if utext.endswith('\n'):
-            txt += linesep
-        return txt
+        for symbol in EOL_SYMBOLS:
+            text = text.replace(symbol, linesep)
+        return text
 
     #------Positions, coordinates (cursor, EOF, ...)
     def get_position(self, subject):
@@ -842,12 +860,6 @@ class BaseEditMixin(object):
 
 
     #------Selection
-    def clear_selection(self):
-        """Clear current selection"""
-        cursor = self.textCursor()
-        cursor.clearSelection()
-        self.setTextCursor(cursor)
-
     def extend_selection_to_next(self, what='word', direction='left'):
         """
         Extend selection to next *what* ('word' or 'character')
@@ -886,18 +898,26 @@ class BaseEditMixin(object):
         start_cursor.setPosition(end_position, mode=QTextCursor.KeepAnchor)
         return self.get_selected_text(start_cursor)
 
-    def get_text(self, position_from, position_to):
-        """
-        Return text between *position_from* and *position_to*
-        Positions may be positions or 'sol', 'eol', 'sof', 'eof' or 'cursor'
+    def get_text(self, position_from, position_to, remove_newlines=True):
+        """Returns text between *position_from* and *position_to*.
+
+        Positions may be integers or 'sol', 'eol', 'sof', 'eof' or 'cursor'.
+
+        Unless position_from='sof' and position_to='eof' any trailing newlines
+        in the string are removed. This was added as a workaround for
+        spyder-ide/spyder#1546 and later caused spyder-ide/spyder#14374.
+        The behaviour can be overridden by setting the optional parameter
+        *remove_newlines* to False.
+
+        TODO: Evaluate if this is still a problem and if the workaround can
+              be moved closer to where the problem occurs.
         """
         cursor = self.__select_text(position_from, position_to)
         text = to_text_string(cursor.selectedText())
-        all_text = position_from == 'sof' and position_to == 'eof'
-        if text and not all_text:
-            while text.endswith("\n"):
-                text = text[:-1]
-            while text.endswith(u"\u2029"):
+        if remove_newlines:
+            remove_newlines = position_from != 'sof' or position_to != 'eof'
+        if text and remove_newlines:
+            while text and text[-1] in EOL_SYMBOLS:
                 text = text[:-1]
         return text
 
@@ -922,6 +942,7 @@ class BaseEditMixin(object):
             self.textCursor().insertText(text)
             if self.sig_text_was_inserted is not None:
                 self.sig_text_was_inserted.emit()
+            self.document_did_change()
 
     def replace_text(self, position_from, position_to, text):
         cursor = self.__select_text(position_from, position_to)
@@ -934,6 +955,7 @@ class BaseEditMixin(object):
         cursor.insertText(text)
         if self.sig_text_was_inserted is not None:
             self.sig_text_was_inserted.emit()
+        self.document_did_change()
 
     def remove_text(self, position_from, position_to):
         cursor = self.__select_text(position_from, position_to)
@@ -941,6 +963,50 @@ class BaseEditMixin(object):
             start, end = self.get_selection_start_end(cursor)
             self.sig_will_remove_selection.emit(start, end)
         cursor.removeSelectedText()
+        self.document_did_change()
+
+    def get_current_object(self):
+        """
+        Return current object under cursor.
+
+        Get the text of the current word plus all the characters
+        to the left until a space is found. Used to get text to inspect
+        for Help of elements following dot notation for example
+        np.linalg.norm
+        """
+        cursor = self.textCursor()
+        cursor_pos = cursor.position()
+        current_word = self.get_current_word(help_req=True)
+
+        # Get max position to the left of cursor until space or no more
+        # characters are left
+        cursor.movePosition(QTextCursor.PreviousCharacter)
+        while self.get_character(cursor.position()).strip():
+            cursor.movePosition(QTextCursor.PreviousCharacter)
+            if cursor.atBlockStart():
+                break
+        cursor_pos_left = cursor.position()
+
+        # Get max position to the right of cursor until space or no more
+        # characters are left
+        cursor.setPosition(cursor_pos)
+        while self.get_character(cursor.position()).strip():
+            cursor.movePosition(QTextCursor.NextCharacter)
+            if cursor.atBlockEnd():
+                break
+        cursor_pos_right = cursor.position()
+
+        # Get text of the object under the cursor
+        current_text = self.get_text(
+            cursor_pos_left, cursor_pos_right).strip()
+        current_object = current_word
+
+        if current_text and current_word is not None:
+            if current_word != current_text and current_word in current_text:
+                current_object = (
+                    current_text.split(current_word)[0] + current_word)
+
+        return current_object
 
     def get_current_word_and_position(self, completion=False, help_req=False,
                                       valid_python_variable=True):
@@ -971,14 +1037,18 @@ class BaseEditMixin(object):
                 return not to_text_string(curs.selectedText()).strip()
 
             def is_special_character(move):
-                    curs = self.textCursor()
-                    curs.movePosition(move, QTextCursor.KeepAnchor)
-                    text_cursor = to_text_string(curs.selectedText()).strip()
-                    return len(re.findall(r'([^\d\W]\w*)',
-                                          text_cursor, re.UNICODE)) == 0
+                """Check if a character is a non-letter including numbers."""
+                curs = self.textCursor()
+                curs.movePosition(move, QTextCursor.KeepAnchor)
+                text_cursor = to_text_string(curs.selectedText()).strip()
+                return len(
+                    re.findall(r'([^\d\W]\w*)', text_cursor, re.UNICODE)) == 0
+
             if help_req:
-                if is_special_character(QTextCursor.NoMove):
-                    cursor.movePosition(QTextCursor.WordLeft)
+                if is_special_character(QTextCursor.PreviousCharacter):
+                    cursor.movePosition(QTextCursor.NextCharacter)
+                elif is_special_character(QTextCursor.NextCharacter):
+                    cursor.movePosition(QTextCursor.PreviousCharacter)
             elif not completion:
                 if is_space(QTextCursor.NextCharacter):
                     if is_space(QTextCursor.PreviousCharacter):
@@ -1057,11 +1127,15 @@ class BaseEditMixin(object):
 
         return word
 
+    def get_line_indentation(self, text):
+        """Get indentation for given line."""
+        text = text.replace("\t", " "*self.tab_stop_width_spaces)
+        return len(text)-len(text.lstrip())
+
     def get_block_indentation(self, block_nb):
         """Return line indentation (character number)."""
         text = to_text_string(self.document().findBlockByNumber(block_nb).text())
-        text = text.replace("\t", " "*self.tab_stop_width_spaces)
-        return len(text)-len(text.lstrip())
+        return self.get_line_indentation(text)
 
     def get_selection_bounds(self, cursor=None):
         """Return selection bounds (block numbers)."""
@@ -1105,6 +1179,13 @@ class BaseEditMixin(object):
     def remove_selected_text(self):
         """Delete selected text."""
         self.textCursor().removeSelectedText()
+        # The next three lines are a workaround for a quirk of
+        # QTextEdit. See spyder-ide/spyder#12663 and
+        # https://bugreports.qt.io/browse/QTBUG-35861
+        cursor = self.textCursor()
+        cursor.setPosition(cursor.position())
+        self.setTextCursor(cursor)
+        self.document_did_change()
 
     def replace(self, text, pattern=None):
         """Replace selected text by *text*.
@@ -1128,6 +1209,7 @@ class BaseEditMixin(object):
         if self.sig_text_was_inserted is not None:
             self.sig_text_was_inserted.emit()
         cursor.endEditBlock()
+        self.document_did_change()
 
 
     #------Find/replace
@@ -1163,10 +1245,16 @@ class BaseEditMixin(object):
 
     def find_text(self, text, changed=True, forward=True, case=False,
                   word=False, regexp=False):
-        """Find text"""
+        """Find text."""
         cursor = self.textCursor()
         findflag = QTextDocument.FindFlag()
 
+        # Get visible region to center cursor in case it's necessary.
+        if getattr(self, 'get_visible_block_numbers', False):
+            current_visible_region = self.get_visible_block_numbers()
+        else:
+            current_visible_region = None
+
         if not forward:
             findflag = findflag | QTextDocument.FindBackward
 
@@ -1191,16 +1279,10 @@ class BaseEditMixin(object):
         else:
             text = re.escape(to_text_string(text))
 
-        if QT55_VERSION:
-            pattern = QRegularExpression(u"\\b{}\\b".format(text) if word else
-                                         text)
-            if case:
-                pattern.setPatternOptions(
-                    QRegularExpression.CaseInsensitiveOption)
-        else:
-            pattern = QRegExp(u"\\b{}\\b".format(text)
-                              if word else text, Qt.CaseSensitive if case else
-                              Qt.CaseInsensitive, QRegExp.RegExp2)
+        pattern = QRegularExpression(u"\\b{}\\b".format(text) if word else
+                                     text)
+        if case:
+            pattern.setPatternOptions(QRegularExpression.CaseInsensitiveOption)
 
         for move in moves:
             cursor.movePosition(move)
@@ -1214,6 +1296,14 @@ class BaseEditMixin(object):
                 found_cursor = self.document().find(pattern, cursor, findflag)
             if found_cursor is not None and not found_cursor.isNull():
                 self.setTextCursor(found_cursor)
+
+                # Center cursor if we move out of the visible region.
+                if current_visible_region is not None:
+                    found_visible_region = self.get_visible_block_numbers()
+                    if current_visible_region != found_visible_region:
+                        current_visible_region = found_visible_region
+                        self.centerCursor()
+
                 return True
 
         return False
@@ -1276,9 +1366,9 @@ class BaseEditMixin(object):
 
         # TODO: adapt to font size
         x = rect.left()
-        x = x - 14
+        x = int(x - 14)
         y = rect.top() + (rect.bottom() - rect.top())/2
-        y = y - dlg.height()/2 - 3
+        y = int(y - dlg.height()/2 - 3)
 
         pos = QPoint(x, y)
         pos = self.calculate_real_position(pos)
@@ -1304,12 +1394,16 @@ class BaseEditMixin(object):
                 if self.sig_text_was_inserted is not None:
                     self.sig_text_was_inserted.emit()
                 cursor.endEditBlock()
+                self.document_did_change()
 
 
 class TracebackLinksMixin(object):
     """ """
     QT_CLASS = None
-    go_to_error = None
+
+    # This signal emits a parsed error traceback text so we can then
+    # request opening the file that traceback comes from in the Editor.
+    sig_go_to_error_requested = None
 
     def __init__(self):
         self.__cursor_changed = False
@@ -1321,8 +1415,8 @@ class TracebackLinksMixin(object):
         self.QT_CLASS.mouseReleaseEvent(self, event)
         text = self.get_line_at(event.pos())
         if get_error_match(text) and not self.has_selected_text():
-            if self.go_to_error is not None:
-                self.go_to_error.emit(text)
+            if self.sig_go_to_error_requested is not None:
+                self.sig_go_to_error_requested.emit(text)
 
     def mouseMoveEvent(self, event):
         """Show Pointing Hand Cursor on error messages"""
@@ -1347,21 +1441,17 @@ class TracebackLinksMixin(object):
 
 
 class GetHelpMixin(object):
-    def __init__(self):
-        self.help = None
-        self.help_enabled = False
 
-    def set_help(self, help_plugin):
-        """Set Help DockWidget reference"""
-        self.help = help_plugin
+    def __init__(self):
+        self.help_enabled = False
 
     def set_help_enabled(self, state):
         self.help_enabled = state
 
     def inspect_current_object(self):
-        text = self.get_current_word(help_req=True)
-        if text:
-            self.show_object_info(text, force=True)
+        current_object = self.get_current_object()
+        if current_object is not None:
+            self.show_object_info(current_object, force=True)
 
     def show_object_info(self, text, call=False, force=False):
         """Show signature calltip and/or docstring in the Help plugin"""
@@ -1369,21 +1459,12 @@ class GetHelpMixin(object):
 
         # Show docstring
         help_enabled = self.help_enabled or force
-        if force and self.help is not None:
-            self.help.dockwidget.setVisible(True)
-            self.help.dockwidget.raise_()
-        if help_enabled and (self.help is not None) and \
-           (self.help.dockwidget.isVisible()):
-            # Help widget exists and is visible
-            if hasattr(self, 'get_doc'):
-                self.help.set_shell(self)
-            else:
-                self.help.set_shell(self.parent())
-            self.help.set_object_text(text, ignore_unknown=False)
-            self.setFocus() # if help was not at top level, raising it to
-                            # top will automatically give it focus because of
-                            # the visibility_changed signal, so we must give
-                            # focus back to shell
+        if help_enabled:
+            doc = {
+                'name': text,
+                'ignore_unknown': False,
+            }
+            self.sig_help_requested.emit(doc)
 
         # Show calltip
         if call and getattr(self, 'calltips', None):
@@ -1428,7 +1509,7 @@ class SaveHistoryMixin(object):
     SEPARATOR = None
     HISTORY_FILENAMES = []
 
-    append_to_history = None
+    sig_append_to_history_requested = None
 
     def __init__(self, history_filename=''):
         self.history_filename = history_filename
@@ -1466,8 +1547,9 @@ class SaveHistoryMixin(object):
             encoding.write(text, self.history_filename, mode='ab')
         except EnvironmentError:
             pass
-        if self.append_to_history is not None:
-            self.append_to_history.emit(self.history_filename, text)
+        if self.sig_append_to_history_requested is not None:
+            self.sig_append_to_history_requested.emit(
+                self.history_filename, text)
 
 
 class BrowseHistory(object):
